/*-------------------------------------------------------------------------------
Using Welsh Powel Algorithms..Colouring in order of decreasing degree
Takes in Adjacency List
Author: G.Kavitha
Input format: ./ProgName fileName NoOfVertices
Assuming vertices are numbered 0 to V-1
---------------------------------------------------------------------------------*/
#include<bits/stdc++.h>
using namespace std;
# define l int
struct nodes
{
	l node; 
	l degree;
};
class Graph
{
	l V; //No. of vertices
	list<l> *adj; //Adjacency List
	l *deg; //Holds degree of vertices
	void addEdge(l a,l b);
	void colouring();
public:
	Graph(l ver,char* s)//Constructor
	{
		this->V=ver;
		adj=new list<l>[V];
		deg=new l[V];
		memset(deg,0,sizeof(l)*V);
		ifstream fin;
		fin.open(s,ios::in);
		l a,b;
		while(!fin.eof())
		{
			fin>>a>>b;
			addEdge(a,b);
			deg[a]++;
			deg[b]++;
		}
		fin.close();
		colouring();
	}
	//Destructor
	~Graph()
	{
		delete [] adj;
	}
};

//Function to compare vertices based on degree
bool compare(nodes a,nodes b) 
{
	return(a.degree>b.degree);
}

void Graph::addEdge(l a,l b)
{
	adj[a].push_back(b);
	adj[b].push_back(a);
}

void Graph::colouring()
{
	nodes *vert=new nodes[V];
	for(l v=0;v<V;v++)
	{
		vert[v].node=v; 
		vert[v].degree=deg[v];
	}
	sort(vert,vert+V,compare); //sorts in descending order of degree
	l *color=new l[V]; //Holds the colour assigned to each vertex
	bool *neighbourcolor=new bool[V]; //True when a neighbouring vertex is coloured with that colour
	l max=1;//To hold the maximum number of vertices required
	color[vert[0].node]=1; //Colouring vertex with largest degree with colour 1

	//Initially setting false for every colour 
	for(l c=1;c<=V;c++)
		neighbourcolor[c]=false; 
	for(l v=1;v<V;v++)
		color[vert[v].node]=-1; //No colour initially assigned
	for(l v=1;v<V;v++)
	{
		list<l>::iterator i;
		for(i=adj[vert[v].node].begin();i!=adj[vert[v].node].end();i++)
		{
			if(color[*i]!=-1) 
				neighbourcolor[color[*i]]=true;
		}
		l c;
		for(c=1;c<=V;c++) //Finding first unassigned colour
		{
			if(neighbourcolor[c]==false) break;
		}
		color[vert[v].node]=c;
		if(max<c) max=c;
		for(i=adj[vert[v].node].begin();i!=adj[vert[v].node].end();i++)
		{
			if(color[*i]!=-1) 
				neighbourcolor[color[*i]]=false; //Setting false for all colours again
		}
	}
	for(l v=0;v<V;v++)
	{
		cout<<v<<" - "<<color[v]<<"\n";
	}
	cout<<"No. of colours used: "<<max<<"\n";
}

int main(int argc,char** argv)
{
	clock_t t;
	t=clock();
	if(argc!=3) {cout<<"Usage : ./a.out filename V \n"; return 0;}
	l ver=atoi(argv[2]);
	Graph G(ver,argv[1]);
	t=clock()-t;
	cout<<"Time taken in seconds is: "<<(float(t))/CLOCKS_PER_SEC<<"\n";	
	return 0;
}