/*-------------------------------------------------------------------------------
DSATUR Greedy Algorithm for Graph Colouring(edited)
Takes in Adjacency List
Author: G.Kavitha
Input format: ./ProgName fileName NoOfVertices
Assuming vertices are numbered 0 to V-1
---------------------------------------------------------------------------------*/

//Edit done was that instead of taking saturation degree as number of coloured neighbours,
//I took it as number of uniquely coloured neighbours

#include<bits/stdc++.h>
using namespace std;
# define l int
l n;
class Graph
{
	l V; //No. of vertices
	l *vert; //for deciding the order in which vertices are to be assigned colours
	l *color; //to hold the colour assigned to the vertices
	list<l> *adj; //Adjacency List
	l *deg; //Holds degree of vertices
	map<l,bool> *satdeg; //Edited here------Holds saturation degree(no of unique colours in adjacent vertices)
	void addEdge(l a,l b);
	void colouring();
public:

	Graph(l ver)//Constructor
	{
		this->V=ver;
	}
	void graph(l ver){
		this->V=ver;
		adj=new list<l>[V];
		deg=new l[V];
		satdeg=new map<l,bool>[V];
		vert=new l[V];
		color=new l[V];
		memset(deg,0,sizeof(l)*V);
		memset(color,-1,sizeof(l)*V);//initially no colour is assigned
		for(l i=0;i<V;i++) vert[i]=i;
	}
	void dsatur(l ver,char* s);
	void getvalues(l a,l b,l arr[4]); //function to get required values for the compare() function needed to sort the vertices

	//destructor
	~Graph()
	{
		delete [] adj;
		delete [] deg;
		delete [] satdeg;
		delete [] vert;
	}
};
Graph G(0);
bool compare(l a,l b) //function to sort vertices based on saturatiion degree
	{
		l arr[4];
		G.getvalues(a,b,arr);
		if(arr[0]==arr[1]) return(arr[2]>arr[3]);
		return(arr[0]>arr[1]);
	}

void Graph::dsatur(l ver,char* s)
	{
		ifstream fin;
		fin.open(s,ios::in);
		l a,b;
		while(!fin.eof())
		{
			fin>>a>>b;
			addEdge(a,b);
			deg[a]++;
			deg[b]++;
		}
		fin.close();
		colouring();
	}
void Graph::getvalues(l a,l b,l arr[4])
	{
		arr[0]=satdeg[a].size(); if(color[a]!=-1) arr[0]=INT_MIN;
		arr[1]=satdeg[b].size(); if(color[b]!=-1) arr[1]=INT_MIN;
		arr[2]=deg[a];
		arr[3]=deg[b];
	}
	
void Graph::addEdge(l a,l b)
{
	adj[a].push_back(b);
	adj[b].push_back(a);
}
void Graph::colouring()
{
	bool neighbourcolor[V]; //True when a neighbouring vertex is coloured with that colour
	
	sort(vert,vert+V,compare);//sorts based on dec saturation degree, in case of tie, it sorts based on decreasing degree
	
	color[vert[0]]=1;//colouring vertex with maximum degree

	satdeg[vert[0]].clear();//=INT_MIN; //so that this vertex won't be coloured again

	list<l>::iterator i;
	//Updating satdeg values for adjacent vertices
	for(i=adj[vert[0]].begin();i!=adj[vert[0]].end();i++)
	{
		satdeg[*i][1]=true; 
	}

	l c;
	l max=1;//holds maximum number of colours used
	//Setting false for all colours initially
	for(c=1;c<=V;c++)
		neighbourcolor[c]=false;
	
	for(l j=1;j<V;j++)
	{
		sort(vert,vert+V,compare);
		for(i=adj[vert[0]].begin();i!=adj[vert[0]].end();i++)
		{
			if(color[*i]!=-1) neighbourcolor[color[*i]]=true; //finding used colours
		}
		for(c=1;c<=V;c++)
		{
			if(neighbourcolor[c]==false) break; //finding first unused colour
		}
		color[vert[0]]=c;
		satdeg[vert[0]].clear();//=INT_MIN;
		for(i=adj[vert[0]].begin();i!=adj[vert[0]].end();i++)
		{
			satdeg[*i][c]=true;//Updating saturation degree for adjacent vertices
		}
		if(c>max) max=c;
		for(i=adj[vert[0]].begin();i!=adj[vert[0]].end();i++)
		{
			if(color[*i]!=-1) 
				neighbourcolor[color[*i]]=false; //Setting false for all colours again
		}
	}
	for(l j=0;j<V;j++)
	{
		cout<<j<<" - "<<color[j]<<"\n";
	}
	cout<<"No. of colours used: "<<max<<"\n";
}

int main(int argc,char** argv)
{
	clock_t t;
	t=clock();
	if(argc!=3) {cout<<"Usage : ./a.out filename V \n"; return 0;}
	n=atoi(argv[2]);
	G.graph(n);
	G.dsatur(n,argv[1]);
	t=clock()-t;
	cout<<"Time taken in seconds is: "<<(float(t))/CLOCKS_PER_SEC<<"\n";	
	return 0;
}